<?php
	if ( ! defined( 'ABSPATH' ) ) {
		exit; // Exit if accessed directly.
	}

	if ( ! class_exists( 'WP_List_Table' ) ) {
		require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
	}

	class ZLEADSCRM_PROSPECT_Core_Bookmarks extends WP_List_Table {

		private $user_id;

		private $current_page_number;

		public function __construct( $user_id ) {
			$this->items   = [];
			$this->user_id = $user_id;
			parent::__construct(
				array(
					'singular' => 'bookmark',
					'plural'   => 'bookmarks',
					'ajax'     => false,
				)
			);
		}

		public function print_overview() {
			$this->prepare_items();

			echo '<div class="zleadscrm_overview_wrap">';
			echo '<h2 class="screen-reader-text">Bookmarks List</h2>';

			if ( count( $this->items ) != 0 ) {
				$this->display();
			} else {
				if ( $this->user_id != 0 ) {
					echo '<div style="color:#000; padding:1px 12px; background: #fff; border-left: solid 4px #ffb900;">';
					echo '<p style="margin: .5em 0; padding: 2px;">No Bookmarks - Click the star icon to add the business as a bookmark to view later</p>';
					echo '</div>';
				} else {
					echo '<div style="color:#000; padding:1px 12px; background: #fff; border-left: solid 4px #ffb900;">';
					echo '<p style="margin: .5em 0; padding: 2px;">No Shared Bookmarks - Click the star icon to add the business as a bookmark to view later and share as a team</p>';
					echo '</div>';
				}
			}


			echo '</div>';
		}

		public function column_default( $result, $column_name ) {
			$current_user_id = get_current_user_id();
			if ( $this->user_id == 0 ) {
				if ( zleadscrm_user_can_do_advanced_search( $current_user_id ) ) {
					switch ( $column_name ) {
						case 'business_name':
							return '<span>' . $result['business_name'] . '</span>';
						case 'business_type':
							$o = '';
							if ( is_array( $result['business_type'] ) ) {
								foreach ( $result['business_type'] as $btype ) {
									$o .= '<span>' . ucfirst( str_replace( '_', ' ', $btype ) ) . '</span><br/>';
								}
							} else {
								$arr = explode( ';', $result['business_type'] );
								foreach ( $arr as $btype ) {
									$o .= '<span>' . ucfirst( str_replace( '_', ' ', $btype ) ) . '</span><br/>';
								}
							}

							return $o;
						case 'phone':
							return '<span>' . $result['phone'] . '</span>';
						case 'website':
							return '<a target="_blank" href="' . $result['website'] . '">' . $result['website'] . '</a>';
						case 'address':
							return '<span>' . $result['address'] . '</span>';
						case 'owners':
							$o = '';

							$arr = explode( ';', $result['owners'] );
							foreach ( $arr as $owner ) {
								$o .= '<span>' . $owner . '</span><br/>';
							}

							return $o;
						case 'actions':
							ob_start();
							?><p>
							<?php
							$data = '';
							$data .= str_replace( '#', 'no. ', $result['business_name'] ) . '~';
							$o    = '';
							if ( is_array( $result['business_type'] ) ) {
								foreach ( $result['business_type'] as $btype ) {
									$o .= ucfirst( str_replace( '_', ' ', $btype ) ) . ';';
								}
							} else {
								$o .= $result['business_type'];
							}
							$data  .= $o . '~';
							$data  .= str_replace( '#', '', $result['phone'] ) . '~';
							$data  .= str_replace( '#', '', $result['website'] ) . '~';
							$data  .= str_replace( '#', 'no. ', $result['address'] ) . '~';
							$owner = get_current_user_id() . '~';
							$data  .= $owner;
							$url   = $result['url'];
							$data  .= $url;


							$is_bookmarked = $this->is_bookmarked( $result ) == true ? 'true' : 'false';

							$bookmark = array(
								'data-link'          => admin_url( 'admin-ajax.php' ),
								'data-info'          => $data,
								'data-is-bookmarked' => $is_bookmarked
							);
							printf( '<a class="button dashicons dashicons-star-' . ( $bookmark['data-is-bookmarked'] == 'true' ? 'filled' : 'empty' ) . ' lead_modify_bookmark" data-tip="Bookmark" data-link="%s" data-info="%s" data-is-bookmarked="%s" style="width: 35px;"></a>', esc_url( $bookmark['data-link'] ), $bookmark['data-info'], $bookmark['data-is-bookmarked'] );


							$add_new_url = remove_query_arg( 'tab' );

							$add_new_business_name = str_replace( ' ', '+', $result['business_name'] );
							$add_new_business_name = str_replace( '&', '%amp', $add_new_business_name );

							$add_new_url = add_query_arg( array(
								'add_new'         => 1,
								'from_bookmark'   => 1,
								'business_name'   => $add_new_business_name,
								'mobile'          => $result['phone'],
								'website'         => $result['website'],
								'account_manager' => get_current_user_id(),
								'address'         => $result['address']
							), $add_new_url );


							$actions['add_lead'] = array(
								'url'   => $add_new_url,
								'name'  => 'Add New Lead',
								'class' => 'index-card'
							);

							$actions['maps'] = array(
								'url'    => $url,
								'name'   => 'Maps',
								'action' => 'maps',
								'class'  => 'location'
							);

							$search_link = str_replace( ' ', '+', $result['business_name'] );

							$actions['search'] = array(
								'url'    => 'https://www.google.com/search?q=' . $search_link,
								'name'   => 'Search',
								'action' => 'search',
								'class'  => 'search'
							);

							foreach ( $actions as $action ) {
								printf( '<a class="button tips dashicons dashicons-%s" href="%s" target="_blank" style="width: 35px;"></a>', esc_attr( $action['class'] ), esc_url( $action['url'] ) );
							}
							?>
                            </p>
							<?php
							$result_actions = ob_get_contents();
							ob_end_clean();

							return $result_actions;
					}
				} else {
					if ( zleadscrm_user_can_do_basic_search( $current_user_id ) ) {
						switch ( $column_name ) {
							case 'business_name':
								return '<span>' . $result['business_name'] . '</span>';
							case 'business_type':
								$o = '';
								if ( is_array( $result['business_type'] ) ) {

									foreach ( $result['business_type'] as $btype ) {
										$o .= '<span>' . ucfirst( str_replace( '_', ' ', $btype ) ) . '</span><br/>';
									}
								} else {
									$arr = explode( ';', $result['business_type'] );
									foreach ( $arr as $btype ) {
										$o .= '<span>' . ucfirst( str_replace( '_', ' ', $btype ) ) . '</span><br/>';
									}
								}

								return $o;
							case 'address':
								return '<span>' . $result['address'] . '</span>';

							case 'owners':
								$o = '';

								$arr = explode( ';', $result['owners'] );
								foreach ( $arr as $owner ) {
									$o .= '<span>' . $owner . '</span><br/>';
								}


								return $o;
							case 'actions':
								ob_start();
								?><p>
								<?php

								$data = '';
								$data .= str_replace( '#', 'no. ', $result['business_name'] ) . '~';
								$o    = '';
								if ( is_array( $result['business_type'] ) ) {
									foreach ( $result['business_type'] as $btype ) {
										$o .= ucfirst( str_replace( '_', ' ', $btype ) ) . ';';
									}
								} else {
									$o .= $result['business_type'];
								}

								$data  .= $o . '~';
								$data  .= str_replace( '#', 'no. ', $result['address'] ) . '~';
								$owner = get_current_user_id() . '~';
								$data  .= $owner;
								$url   = $result['url'];
								$data  .= $url;


								$is_bookmarked = $this->is_bookmarked( $result ) == true ? 'true' : 'false';

								$bookmark = array(
									'data-link'          => admin_url( 'admin-ajax.php' ),
									'data-info'          => $data,
									'data-is-bookmarked' => $is_bookmarked
								);
								printf( '<a class="button dashicons dashicons-star-' . ( $bookmark['data-is-bookmarked'] == 'true' ? 'filled' : 'empty' ) . ' lead_modify_bookmark" data-tip="Bookmark" data-link="%s" data-info="%s" data-is-bookmarked="%s" style="width: 35px;"></a>', esc_url( $bookmark['data-link'] ), $bookmark['data-info'], $bookmark['data-is-bookmarked'] );


								$add_new_url = remove_query_arg( 'tab' );

								$add_new_business_name = str_replace( ' ', '+', $result['business_name'] );
								$add_new_business_name = str_replace( '&', '%amp', $add_new_business_name );

								$add_new_url = add_query_arg( array(
									'add_new'         => 1,
									'from_bookmark'   => 1,
									'business_name'   => $add_new_business_name,
									'mobile'          => $result['phone'],
									'website'         => $result['website'],
									'account_manager' => get_current_user_id(),
									'address'         => $result['address']
								), $add_new_url );


								$actions['add_lead'] = array(
									'url'   => $add_new_url,
									'name'  => 'Add New Lead',
									'class' => 'index-card'
								);

								$actions['maps'] = array(
									'url'    => $url,
									'name'   => 'Maps',
									'action' => 'maps',
									'class'  => 'location'
								);

								$search_link = str_replace( ' ', '+', $result['business_name'] );

								$actions['search'] = array(
									'url'    => 'https://www.google.com/search?q=' . $search_link,
									'name'   => 'Search',
									'action' => 'search',
									'class'  => 'search'
								);

								foreach ( $actions as $action ) {
									printf( '<a class="button tips dashicons dashicons-%s" href="%s" target="_blank" style="width: 35px;"></a>', esc_attr( $action['class'] ), esc_url( $action['url'] ) );
								}
								?>
                                </p>
								<?php
								$result_actions = ob_get_contents();
								ob_end_clean();

								return $result_actions;
						}
					}
				}
			} else {
				if ( zleadscrm_user_can_do_advanced_search( $current_user_id ) ) {
					switch ( $column_name ) {
						case 'business_name':
							return '<span>' . $result['business_name'] . '</span>';
						case 'business_type':
							$o = '';
							if ( is_array( $result['business_type'] ) ) {
								foreach ( $result['business_type'] as $btype ) {
									$o .= '<span>' . ucfirst( str_replace( '_', ' ', $btype ) ) . '</span><br/>';
								}
							} else {
								$arr = explode( ';', $result['business_type'] );
								foreach ( $arr as $btype ) {
									$o .= '<span>' . ucfirst( str_replace( '_', ' ', $btype ) ) . '</span><br/>';
								}
							}

							return $o;
						case 'phone':
							return '<span>' . $result['phone'] . '</span>';
						case 'website':
							return '<a target="_blank" href="' . $result['website'] . '">' . $result['website'] . '</a>';
						case 'address':
							return '<span>' . $result['address'] . '</span>';
						case 'actions':
							ob_start();
							?><p>
							<?php
							$data = '';
							$data .= str_replace( '#', 'no. ', $result['business_name'] ) . '~';
							$o    = '';
							if ( is_array( $result['business_type'] ) ) {
								foreach ( $result['business_type'] as $btype ) {
									$o .= ucfirst( str_replace( '_', ' ', $btype ) ) . ';';
								}
							} else {
								$o .= $result['business_type'];
							}
							$data  .= $o . '~';
							$data  .= str_replace( '#', '', $result['phone'] ) . '~';
							$data  .= str_replace( '#', '', $result['website'] ) . '~';
							$data  .= str_replace( '#', 'no. ', $result['address'] ) . '~';
							$owner = get_current_user_id() . '~';
							$data  .= $owner;
							$url   = $result['url'];
							$data  .= $url;


							$is_bookmarked = $this->is_bookmarked( $result ) == true ? 'true' : 'false';

							$bookmark = array(
								'data-link'          => admin_url( 'admin-ajax.php' ),
								'data-info'          => $data,
								'data-is-bookmarked' => $is_bookmarked
							);
							printf( '<a class="button dashicons dashicons-star-' . ( $bookmark['data-is-bookmarked'] == 'true' ? 'filled' : 'empty' ) . ' lead_modify_bookmark" data-tip="Bookmark" data-link="%s" data-info="%s" data-is-bookmarked="%s" style="width: 35px;"></a>', esc_url( $bookmark['data-link'] ), $bookmark['data-info'], $bookmark['data-is-bookmarked'] );

							$add_new_url = remove_query_arg( 'tab' );

							$add_new_business_name = str_replace( ' ', '+', $result['business_name'] );

							$add_new_business_name = str_replace( '&', '%amp', $add_new_business_name );

							$add_new_url = add_query_arg( array(
								'add_new'         => 1,
								'from_bookmark'   => 1,
								'business_name'   => $add_new_business_name,
								'mobile'          => $result['phone'],
								'website'         => $result['website'],
								'account_manager' => get_current_user_id(),
								'address'         => $result['address']
							), $add_new_url );


							$actions['add_lead'] = array(
								'url'   => $add_new_url,
								'name'  => 'Add New Lead',
								'class' => 'index-card'
							);

							$actions['maps'] = array(
								'url'    => $url,
								'name'   => 'Maps',
								'action' => 'maps',
								'class'  => 'location'
							);

							$search_link = str_replace( ' ', '+', $result['business_name'] );

							$actions['search'] = array(
								'url'    => 'https://www.google.com/search?q=' . $search_link,
								'name'   => 'Search',
								'action' => 'search',
								'class'  => 'search'
							);

							foreach ( $actions as $action ) {
								printf( '<a class="button tips dashicons dashicons-%s" href="%s" target="_blank" style="width: 35px;"></a>', esc_attr( $action['class'] ), esc_url( $action['url'] ) );
							}
							?>
                            </p>
							<?php
							$result_actions = ob_get_contents();
							ob_end_clean();

							return $result_actions;
					}
				} else {
					if ( zleadscrm_user_can_do_basic_search( $current_user_id ) ) {
						switch ( $column_name ) {
							case 'business_name':
								return '<span>' . $result['business_name'] . '</span>';
							case 'business_type':
								$o = '';
								if ( is_array( $result['business_type'] ) ) {

									foreach ( $result['business_type'] as $btype ) {
										$o .= '<span>' . ucfirst( str_replace( '_', ' ', $btype ) ) . '</span><br/>';
									}
								} else {
									$arr = explode( ';', $result['business_type'] );
									foreach ( $arr as $btype ) {
										$o .= '<span>' . ucfirst( str_replace( '_', ' ', $btype ) ) . '</span><br/>';
									}
								}

								return $o;
							case 'address':
								return '<span>' . $result['address'] . '</span>';
							case 'actions':
								ob_start();
								?><p>
								<?php

								$data = '';
								$data .= str_replace( '#', 'no. ', $result['business_name'] ) . '~';
								$o    = '';
								if ( is_array( $result['business_type'] ) ) {
									foreach ( $result['business_type'] as $btype ) {
										$o .= ucfirst( str_replace( '_', ' ', $btype ) ) . ';';
									}
								} else {
									$o .= $result['business_type'];
								}

								$data  .= $o . '~';
								$data  .= str_replace( '#', 'no. ', $result['address'] ) . '~';
								$owner = get_current_user_id() . '~';
								$data  .= $owner;
								$url   = $result['url'];
								$data  .= $url;


								$is_bookmarked = $this->is_bookmarked( $result ) == true ? 'true' : 'false';

								$bookmark = array(
									'data-link'          => admin_url( 'admin-ajax.php' ),
									'data-info'          => $data,
									'data-is-bookmarked' => $is_bookmarked
								);
								printf( '<a class="button dashicons dashicons-star-' . ( $bookmark['data-is-bookmarked'] == 'true' ? 'filled' : 'empty' ) . ' lead_modify_bookmark" data-tip="Bookmark" data-link="%s" data-info="%s" data-is-bookmarked="%s" style="width: 35px;"></a>', esc_url( $bookmark['data-link'] ), esc_attr( $bookmark['data-info'] ), esc_attr( $bookmark['data-is-bookmarked'] ) );


								$add_new_url = remove_query_arg( 'tab' );

								$add_new_business_name = str_replace( ' ', '+', esc_attr( $result['business_name'] ) );

								$add_new_business_name = str_replace( '&', '%amp', $add_new_business_name );

								$add_new_url = add_query_arg( array(
									'add_new'         => 1,
									'from_bookmark'   => 1,
									'business_name'   => $add_new_business_name,
									'mobile'          => $result['phone'],
									'website'         => $result['website'],
									'account_manager' => get_current_user_id(),
									'address'         => $result['address']
								), $add_new_url );


								$actions['add_lead'] = array(
									'url'   => $add_new_url,
									'name'  => 'Add New Lead',
									'class' => 'index-card'
								);

								$actions['maps'] = array(
									'url'    => $url,
									'name'   => 'Maps',
									'action' => 'maps',
									'class'  => 'location'
								);

								$search_link = str_replace( ' ', '+', $result['business_name'] );

								$actions['search'] = array(
									'url'    => 'https://www.google.com/search?q=' . $search_link,
									'name'   => 'Search',
									'action' => 'search',
									'class'  => 'search'
								);

								foreach ( $actions as $action ) {
									printf( '<a class="button tips dashicons dashicons-%s" href="%s" target="_blank" style="width: 35px;"></a>', esc_attr( $action['class'] ), esc_url( $action['url'] ) );
								}
								?>
                                </p>
								<?php
								$result_actions = ob_get_contents();
								ob_end_clean();

								return $result_actions;
						}
					}
				}
			}

			return '';
		}

		public function is_bookmarked( $result ) {
			global $wpdb;
			$table_name    = $wpdb->prefix . 'zleadscrm_leads_bookmarks';
			$business_name = $result['business_name'];
			$business_type = '';
			if ( is_array( $result['business_type'] ) ) {
				foreach ( $result['business_type'] as $btype ) {
					$business_type .= ucfirst( str_replace( '_', ' ', $btype ) ) . ';';
				}
			} else {
				$business_type .= $result['business_type'];
			}

			$address = $result['address'];
			$owner   = get_current_user_id();

			$results = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM $table_name WHERE business_name=%s AND business_type=%s AND address=%s AND owner_id=%d LIMIT 1;", $business_name, $business_type, $address, $owner ) );

			if ( count( $results ) != 0 ) {
				return true;
			}

			return false;
		}

		public function display_tablenav( $which ) {
			if ( $which == 'top' ) {
				echo '<div class="zleadscrm_align_table_mobile actions bulkactions">';
				$this->pagination( "top" );
				echo '</div>';
			}
		}

		public function get_columns() {
			$current_user_id = get_current_user_id();
			$columns         = [];
			if ( $this->user_id == 0 ) {
				if ( zleadscrm_user_can_do_advanced_search( $current_user_id ) ) {

					$columns = array(
						'business_name' => 'Business Name',
						'business_type' => 'Business Type',
						'phone'         => 'Phone',
						'website'       => 'Website',
						'address'       => 'Address',
						'owners'        => 'Bookmark Owner',
						'actions'       => 'Actions',
					);
				} else {
					if ( zleadscrm_user_can_do_basic_search( $current_user_id ) ) {
						$columns = array(
							'business_name' => 'Business Name',
							'business_type' => 'Business Type',
							'address'       => 'Address',
							'owners'        => 'Bookmark Owner',
							'actions'       => 'Actions',
						);
					}
				}
			} else {
				if ( zleadscrm_user_can_do_advanced_search( $current_user_id ) ) {

					$columns = array(
						'business_name' => 'Business Name',
						'business_type' => 'Business Type',
						'phone'         => 'Phone',
						'website'       => 'Website',
						'address'       => 'Address',
						'actions'       => 'Actions',
					);
				} else {
					if ( zleadscrm_user_can_do_basic_search( $current_user_id ) ) {
						$columns = array(
							'business_name' => 'Business Name',
							'business_type' => 'Business Type',
							'address'       => 'Address',
							'actions'       => 'Actions',
						);
					}
				}
			}

			return $columns;

		}

		public function prepare_items() {
			$current_page = $this->get_pagenum();
			$per_page     = 20;
			global $wpdb;
			$table_name = $wpdb->prefix . 'zleadscrm_leads_bookmarks';
			/**
			 * Init column headers.
			 */
			$this->_column_headers = array( $this->get_columns(), array(), array() );

			if ( $current_page == 0 ) {
				if ( $this->user_id == 0 ) {
					$results = $wpdb->get_results( "SELECT * FROM $table_name ORDER BY timestamp ASC LIMIT $per_page;" );
				} else {
					$results = $wpdb->get_results( "SELECT * FROM $table_name WHERE owner_id=$this->user_id ORDER BY timestamp ASC LIMIT $per_page;" );
				}
			} else {
				$offset = ( $current_page - 1 ) * $per_page;
				if ( $this->user_id == 0 ) {
					$results = $wpdb->get_results( "SELECT * FROM $table_name ORDER BY timestamp ASC LIMIT $per_page OFFSET $offset;" );
				} else {
					$results = $wpdb->get_results( "SELECT * FROM $table_name WHERE owner_id=$this->user_id ORDER BY timestamp ASC LIMIT $per_page OFFSET $offset;" );
				}
			}
			$arr = [];

			foreach ( $results as $result ) {
				$owners        = '';
				$business_name = $result->business_name;
				$business_type = $result->business_type;
				$address       = $result->address;
				$url           = $result->url;
				if ( $this->user_id == 0 ) {
					$current_user  = $result->owner_id;
					$owners        .= get_user_by( 'id', $current_user )->display_name;
					$other_results = $wpdb->get_results( $wpdb->prepare( "SELECT * FROM $table_name WHERE business_name=%s AND business_type=%s AND address=%s AND owner_id!=%d;", $business_name, $business_type, $address, $current_user ) );
					if ( count( $other_results ) != 0 ) {
						foreach ( $other_results as $other_result ) {
							$owners .= ';' . get_user_by( 'id', $other_result->owner_id )->display_name;
						}
					}
				} else {
					$owners .= get_user_by( 'id', $this->user_id )->display_name;
				}

				$owners_arr = explode( ";", $owners );
				sort( $owners_arr );
				$owners_sorted = implode( ";", $owners_arr );

				array_push( $arr, array(
					'id'            => $result->id,
					'business_name' => $result->business_name,
					'business_type' => $result->business_type,
					'phone'         => $result->phone,
					'website'       => $result->website,
					'address'       => $result->address,
					'owners'        => $owners_sorted,
					'url'           => $url
				) );
			}

			$array = [];

			foreach ( $arr as $item ) {
				if ( ! $this->exist_in_array( $array, $item ) ) {
					array_push( $array, $item );
				}
			}

			$this->items = $array;


			/**
			 * Pagination.
			 */

			$total_results = $this->get_total_number_of_results();

			$this->set_pagination_args(
				array(
					'total_items' => $total_results,
					'per_page'    => $per_page,
					'total_pages' => ceil( $total_results / $per_page ),
				)
			);
		}

		public function get_total_number_of_results() {
			global $wpdb;
			$table_name = $wpdb->prefix . 'zleadscrm_leads_bookmarks';

			if ( $this->user_id == 0 ) {
				$result = $wpdb->get_results( "SELECT DISTINCT business_name, business_type, phone, website, address AS total FROM $table_name;" );

				return count( $result );
			} else {
				$result = $wpdb->get_results( "SELECT COUNT(*) AS total FROM $table_name WHERE owner_id=$this->user_id;" );

				return $result[0]->total;

			}

		}

		private function exist_in_array( $array, $item ) {
			foreach ( $array as $arr ) {
				if ( ( $arr['business_name'] == $item['business_name'] ) &&
				     ( $arr['business_type'] == $item['business_type'] ) &&
				     ( $arr['address'] == $item['address'] ) &&
				     ( $arr['url'] == $item['url'] ) ) {
					return true;
				}
			}

			return false;
		}
	}

?>